# -*- coding: utf-8 -*-
from django.conf.urls import url
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe

from django.utils.translation import ugettext as _
from django.contrib import admin
from django.contrib import messages

from jmb.jadmin import JumboModelAdmin, ConstrainedModelForm

from isolmant.organisation import models
from isolmant.organisation.admin_views import message_preview, sync_message_with_mailup
from isolmant.organisation.admin_forms import NewsletterActionForm
from isolmant.organisation.task import NewsLetterSendTask
from modeltranslation.admin import TranslationAdmin

log_fieldset = (
    _('logs admin'), {
        'fields': (
            'creator',
            'date_create',
            'last_modifier',
            'date_last_modify',
            'status',
        ),
        'classes': ('collapse',)
    }
)


class ContactAdmin(JumboModelAdmin):
    advanced_search_fields = (
        ('source',),
        ('contact_category',),
        ('purchase_category',),
        ('last_name__icontains', 'first_name__icontains', 'organisation_name__icontains',),
        ('city__icontains', 'province__icontains', 'country__icontains',),
    )
    fieldsets = (
        log_fieldset,
        (_('contact information'), {
            'fields': (
                'email',
                ('first_name', 'last_name',),
                ('organisation_name', 'role',),
                'brand',
                'ssn',
                'vat',
                'phone',
                'mobile',
                'fax',
                'url',
            )
        }),
        (_('address'), {
           'fields': (
               'address',
               'city',
               'zip_code',
               'province',
               'country'
           )
        }),
        (_('crm'), {
            'fields': (
                'source',
                'contact_category',
                'purchase_category',
                'note'
            )
        }),
        (_('mailup'), {
            'fields': (
                'newsletter',
                'mailup_list_id',
                'mailup_recipient_id'
            )
        })
    )
    list_display = (
        'get_edit_icon',
        'email',
        'first_name',
        'last_name',
        'source',
        'contact_category',
        'get_purchase_category_list',
        'country',
        'get_delete_icon',
    )

    prepopulated_fields = {
        'vat': ('ssn',),
    }

    def get_purchase_category_list(self, instance):
        categories = ''
        for category in instance.purchase_category.all():
            categories += '{}, '.format(category)
        return categories
    get_purchase_category_list.short_description = _('Purchase category')

    def newsletter_send_task(self, request, contacts):
        message_id = request.POST['newsletter']
        NewsLetterSendTask.delay_or_fail(
            message_id=message_id,
            contacts=contacts,
        )
        messages.success(request, _('Newsletter send task created, emails will be sent in a few minutes'))

    action_form = NewsletterActionForm
    actions = [newsletter_send_task]


class ContactSourceAdmin(JumboModelAdmin):
    fieldsets = (
        log_fieldset,
        (_('main'), {
            'fields': (
                'name',
                'description',
            )
        }),
    )
    list_display = (
        'get_edit_icon',
        'name',
        'get_delete_icon'
    )


class ContactCategoryAdmin(JumboModelAdmin, TranslationAdmin):
    fieldsets = (
        log_fieldset,
        (_('main'), {
            'fields': (
                'name',
                'description',
            )
        }),
    )
    list_display = (
        'get_edit_icon',
        'name',
        'get_delete_icon'
    )


class PurchaseCategoryAdmin(JumboModelAdmin, TranslationAdmin):
    fieldsets = (
        log_fieldset,
        (_('main'), {
            'fields': (
                'name',
                'description',
            )
        }),
    )
    list_display = (
        'get_edit_icon',
        'name',
        'get_delete_icon'
    )


class AgentAdmin(JumboModelAdmin):
    fieldsets = (
        log_fieldset,
        (_('agent information'), {
            'fields': (
                'first_name',
                'last_name',
                'typology',
                'organisation',
                'email',
                'phone',
                'mobile',
                'fax',
                'address',
                'provinces'
            )
        }),
    )
    list_display = (
        'get_edit_icon',
        'first_name',
        'last_name',
        'typology',
        'organisation',
        'phone',
        'mobile',
        'email',
        'fax',
        'get_delete_icon',
    )


class ProvinceAdmin(JumboModelAdmin):
    fieldsets = (
        log_fieldset,
        (_('contact information'), {
            'fields': (
                'name',
                'slug',
                'region'
            )
        }),
    )
    list_display = (
        'get_edit_icon',
        'name',
        'slug',
        'region',
        'get_delete_icon',
    )
    prepopulated_fields = {
        'slug': ('name',),
    }


class RegionAdmin(JumboModelAdmin):
    fieldsets = (
        log_fieldset,
        (_('contact information'), {
            'fields': (
                'name',
                'slug',
            )
        }),
    )
    list_display = (
        'get_edit_icon',
        'name',
        'slug',
        'get_delete_icon',
    )
    prepopulated_fields = {
        'slug': ('name',),
    }


class MailUpMessageAdmin(JumboModelAdmin):
    readonly_fields = ('message_id',)
    fieldsets = (
        log_fieldset,
        (_('Message configuration'), {
            'fields': (
                'message_id', 'subject',
            )
        }),
    )
    list_display = (
        'get_edit_icon',
        'get_link_preview', 'message_id', 'subject',
        'get_delete_icon',
    )

    def get_link_preview(self, obj):
        django_message = models.MailUpMessage.objects.get(pk=obj.id)
        return """
            <a href="javascript:;" onClick="window.open('{url}', '{title}', 'width=800, height=600, resizable, status, scrollbars=1, location');">
                <img src="/static/img/preview.png" alt="{alt}">
            </a>
        """.format(
            url=reverse('admin:message_preview', kwargs={'message_id': django_message.message_id}),
            title=_('Preview'),
            alt=_('Message preview'),
        )
    get_link_preview.short_description = _('Preview')
    get_link_preview.allow_tags = True

    def get_urls(self):
        urls = super(MailUpMessageAdmin, self).get_urls()
        cer_codes_detail_url = [
            url(
                r'^get_message_preview/(?P<message_id>\d+)$',
                self.admin_site.admin_view(message_preview),
                name='message_preview',
            ),
            url(
                r'^sync_message_with_mailup/$',
                self.admin_site.admin_view(sync_message_with_mailup),
                name='sync_message_with_mailup',
            ),
        ]
        return cer_codes_detail_url + urls


admin.site.register(models.Contact, ContactAdmin)
admin.site.register(models.ContactSource, ContactSourceAdmin)
admin.site.register(models.ContactCategory, ContactCategoryAdmin)
admin.site.register(models.PurchaseCategory, PurchaseCategoryAdmin)
admin.site.register(models.Agent, AgentAdmin)
admin.site.register(models.Province, ProvinceAdmin)
admin.site.register(models.Region, RegionAdmin)
admin.site.register(models.MailUpMessage, MailUpMessageAdmin)
