# coding: utf-8

from django.conf import settings

from django.db import models
from django.utils.translation import ugettext_lazy as _

from jmb.core.db.models import DateModel, UserModel, StatusModel

from isolmant.organisation import utils


class ContactSource(DateModel, UserModel, StatusModel):
    name = models.CharField(
        _('name'),
        max_length=30,
    )
    description = models.TextField(
        _('description'),
        blank=True,
    )

    class Meta:
        verbose_name = _("contact source")
        verbose_name_plural = _("contact sources")
        ordering = ('name',)

    def __unicode__(self):
        return self.name


class ContactCategory(DateModel, UserModel, StatusModel):
    name = models.CharField(
        _('name'),
        max_length=30,
    )
    description = models.TextField(
        _('description'),
        blank=True,
    )

    class Meta:
        verbose_name = _("contact category")
        verbose_name_plural = _("contact categories")
        ordering = ('name',)

    def __unicode__(self):
        return self.name


class PurchaseCategory(DateModel, UserModel, StatusModel):
    name = models.CharField(
        _('name'),
        max_length=30,
    )
    description = models.TextField(
        _('description'),
        blank=True,
    )

    class Meta:
        verbose_name = _("purchase category")
        verbose_name_plural = _("purchase categories")
        ordering = ('name',)

    def __unicode__(self):
        return self.name


class Contact(DateModel, UserModel, StatusModel):
    """
    Isolmant.Contact model
    """
    # email is the only required and unique field
    email = models.EmailField(
        verbose_name=_('email address'),
    )

    # contact personal info
    first_name = models.CharField(
        verbose_name=_('first name'),
        max_length=30,
        blank=True,
    )
    last_name = models.CharField(
        verbose_name=_('last name'),
        max_length=30,
        blank=True,
    )

    # contact organisation info
    organisation_name = models.CharField(
        verbose_name=_('organisation name'),
        max_length=255,
        blank=True,
    )
    role = models.CharField(
        verbose_name=_('role'),
        max_length=255,
        blank=True,
    )
    brand = models.CharField(
        verbose_name=_('brand'),
        max_length=255,
        blank=True,
    )
    ssn = models.CharField(
        verbose_name=_('social security number'),
        max_length=16,
        blank=True,
    )
    vat = models.CharField(
        verbose_name=_('vat number'),
        max_length=24,
        blank=True,
    )

    # contact information
    phone = models.CharField(
        verbose_name=_('phone number'),
        max_length=30,
        blank=True,
    )
    mobile = models.CharField(
        verbose_name=_('mobile number'),
        max_length=30,
        blank=True,
    )
    fax = models.CharField(
        verbose_name=_("fax"),
        max_length=20,
        blank=True,
    )
    url = models.URLField(
        verbose_name=_("url"),
        max_length=100,
        blank=True,
    )

    # address
    address = models.CharField(
        verbose_name=_('address'),
        max_length=255,
        blank=True,
    )
    city = models.CharField(
        verbose_name=_('city'),
        max_length=255,
        blank=True,
    )
    zip_code = models.CharField(
        verbose_name=_('zip code'),
        max_length=6,
        blank=True,
    )
    province = models.CharField(
        verbose_name=_("province"),
        max_length=100,
        blank=True,
    )
    country = models.CharField(
        verbose_name=_('country'),
        max_length=100,
        blank=True,
    )

    # internal notes

    # crm
    source = models.ForeignKey(
        ContactSource,
        verbose_name=_('contact source'),
        related_name='contacts',
        blank=True, null=True,
    )
    contact_category = models.ForeignKey(
        ContactCategory,
        verbose_name=_('organisation category'),
        related_name='contacts',
        blank=True, null=True,
    )
    purchase_category = models.ManyToManyField(
        PurchaseCategory,
        verbose_name=_('purchase category'),
        related_name='contacts',
        blank=True,
    )
    note = models.TextField(
        verbose_name=_("note"),
        blank=True, null=True,
    )

    # mailup
    newsletter = models.BooleanField(
        verbose_name=_("newsletter"),
        default=True
    )
    mailup_list_id = models.IntegerField(
        verbose_name=_("mailup list id"),
        null=True, blank=True
    )
    mailup_recipient_id = models.IntegerField(
        verbose_name=_("mailup recipient id"),
        null=True, blank=True
    )

    # MAILUP METHODS
    def mailup_get_recipient(self):
        """
        Retrieve MailUp Recipient of contact.email in default list
        """

        mailup_provider = utils.get_default_mailup_provider()
        mailup_list = utils.get_default_mailup_list()

        # RETRIEVE RECIPIENT
        email = self.email
        recipient = mailup_provider.get_recipient(
            list_id=mailup_list.id,
            email=email,
        )
        return recipient

    def mailup_create_recipient(self, confirm_email=True):
        """
        - confirm_email=True: recipient is create in MailUp with status=PENDING and confirmation email is sent.
                              If contact click on confirm link status become SUBSCRIBE

        - confirm_email=False: recipient is create in MailUp with status=SUBSCRIBE
        """

        mailup_list = utils.get_default_mailup_list()

        # RECIPIENT CREATION
        email = self.email
        name = '{first_name} {last_name}'.format(
            first_name=self.first_name,
            last_name=self.last_name,
        )
        recipient_data_dict = {
            'Email': email,
            'Name': name,
            'idList': mailup_list.id,
        }
        mailup_provider = utils.get_default_mailup_provider()
        recipient = mailup_provider.create_recipient(
            data_dict=recipient_data_dict,
            confirm_email=confirm_email,
        )

        # RETRIEVE FIELDS
        recipient = mailup_provider.get_recipient(
            list_id=mailup_list.id,
            recipient_id=recipient.id
        )

        # FIELDS UPDATE
        fields = recipient.get_fields()
        fields['FirstName'] = self.first_name
        fields['LastName'] = self.last_name
        recipient.set_fields(fields)

        # MODEL UPDATE
        self.mailup_recipient_id = recipient.id
        self.mailup_list_id = mailup_list.id
        self.save()

        # SAVE RECIPIENT ON MAILUP
        recipient.save()
        return recipient

    def mailup_subscribe(self, confirm_email=False):

        if self.newsletter:

            mailup_list = utils.get_default_mailup_list()
            recipient = self.mailup_get_recipient()

            import_id = mailup_list.subscribe_recipients_list(
                recipients=[recipient],
                confirm_email=confirm_email,
            )
            if import_id:
                return True
        return False

    def mailup_unsubscribe(self):
        mailup_list = utils.get_default_mailup_list()
        recipient = self.mailup_get_recipient()

        import_id = mailup_list.unsubscribe_recipients_list(
            recipients=[recipient],
        )
        if import_id:
            return True
        return False

    class Meta:
        verbose_name = _("contact")
        verbose_name_plural = _("contacts")
        ordering = ('organisation_name', 'last_name', 'first_name')

    def __unicode__(self):
        if self.organisation_name:
            if not self.last_name:
                return u'{0}'.format(self.organisation_name)
            else:
                if self.role:
                    return u'{0} {1} ({2} - {3})'.format(self.first_name, self.last_name, self.role, self.organisation_name)
                return u'{0} {1} ({2})'.format(self.first_name, self.last_name, self.organisation_name)
        return u'{0} {1}'.format(self.first_name, self.last_name)


class Agent(DateModel, UserModel, StatusModel):
    """
    Agent model
    """
    first_name = models.CharField(
        verbose_name=_('first name'),
        max_length=30,
    )
    last_name = models.CharField(
        verbose_name=_('last name'),
        max_length=30,
    )
    email = models.EmailField(
        verbose_name=_('email'),
        blank=True,
    )
    phone = models.CharField(
        verbose_name=_('phone'),
        max_length=30,
        blank=True,
    )
    mobile = models.CharField(
        verbose_name=_('mobile'),
        max_length=30,
        blank=True,
    )
    fax = models.CharField(
        verbose_name=_('fax'),
        max_length=30,
        blank=True,
    )
    organisation = models.CharField(
        verbose_name=_('organisation'),
        max_length=255,
        blank=True,
    )
    address = models.CharField(
        verbose_name=_('address'),
        max_length=255,
        blank=True,
    )
    provinces = models.ManyToManyField(
        'Province',
        related_name='agents',
        verbose_name=_('Provinces'),
        blank=True,
    )
    typology = models.CharField(
        verbose_name=_('Typology'),
        max_length=128,
        choices=settings.ISOLMANT_ORGANISATION_AGENT_TYPOLOGY_CHOICES,
        default='building'
    )

    class Meta:
        verbose_name = _("agent")
        verbose_name_plural = _("agents")
        ordering = ('last_name', 'first_name')

    def __unicode__(self):
        return u'{0} {1}'.format(self.first_name, self.last_name)

    def get_regions(self):
        region_list = []
        for province in self.provinces.all():
            if province.region.name not in region_list:
                region_list.append(province.region.name)
        return region_list

    def get_regions_slugs(self):
        region_list = []
        for province in self.provinces.all():
            if province.region.slug not in region_list:
                region_list.append(province.region.slug)
        return ' '.join(region_list)


class BuildingAgentManager(models.Manager):
    def get_queryset(self):
        return Agent.objects.filter(
                typology__icontains='building'
        ).prefetch_related(
                'provinces', 'provinces__region'
        ).order_by(
            'organisation', 'last_name', 'first_name'
        )


class BuildingAgent(Agent):
    class Meta:
        verbose_name = _("building agent")
        verbose_name_plural = _("building agents")
        proxy = True

    objects = BuildingAgentManager()


class ParquetAgentManager(models.Manager):
    def get_queryset(self):
        return Agent.objects.filter(
                typology__icontains='parquet'
        ).prefetch_related(
                'provinces', 'provinces__region'
        ).order_by(
            'organisation', 'last_name', 'first_name'
        )


class ParquetAgent(Agent):
    class Meta:
        verbose_name = _("parquet agent")
        verbose_name_plural = _("parquet agents")
        proxy = True

    objects = ParquetAgentManager()


class Province(DateModel, UserModel, StatusModel):
    name = models.CharField(
        verbose_name=_('Name'),
        max_length=255,
        blank=True,
    )
    slug = models.SlugField(
        max_length=255,
        verbose_name=_('slug'),
        unique=True
    )
    region = models.ForeignKey(
        'Region',
        related_name='provinces',
        verbose_name=_('Region'),
        blank=True, null=True,
    )

    class Meta:
        verbose_name = _("province")
        verbose_name_plural = _("provinces")
        ordering = ('region__name', 'name')

    def __unicode__(self):
        return u'{0}'.format(self.name)


class Region(DateModel, UserModel, StatusModel):
    name = models.CharField(
        verbose_name=_('Name'),
        max_length=255,
        blank=True,
    )
    slug = models.SlugField(
        max_length=255,
        verbose_name=_('slug'),
        unique=True
    )

    class Meta:
        verbose_name = _("region")
        verbose_name_plural = _("regions")
        ordering = ('name', )

    def __unicode__(self):
        return u'{0}'.format(self.name)


class MailUpMessage(DateModel, UserModel, StatusModel):
    message_id = models.PositiveIntegerField(
        unique=True,
        verbose_name=_('message id'),
    )
    subject = models.CharField(
        verbose_name=_('subject'),
        max_length=255,
        blank=True,
    )

    class Meta:
        verbose_name = _("MailUp Message")
        verbose_name_plural = _("MailUp Messages")

    def __unicode__(self):
        return u'{0}'.format(self.subject)

    def save(self, mailup_sync=True, *args, **kwargs):
        from mailup import mailup_exceptions

        if mailup_sync:
            mailup_provider = utils.get_default_mailup_provider()
            mailup_list = utils.get_default_mailup_list()

            try:
                message = mailup_provider.get_message(
                    list_id=mailup_list.id,
                    message_id=self.message_id,
                )
                message.subject = self.subject
                message.save()
            except mailup_exceptions.MessageNotFoundException:
                message = mailup_provider.create_message(
                    message_data_dict={
                        'Subject': self.subject,
                        'idList': mailup_list.id,
                    }
                )
                self.message_id = message.id
        super(MailUpMessage, self).save(*args, **kwargs)

    def get_preview(self):
        mailup_provider = utils.get_default_mailup_provider()
        mailup_list = utils.get_default_mailup_list()

        mailup_message = mailup_provider.get_message(
            list_id=mailup_list.id,
            message_id=self.message_id,
        )
        return mailup_message.content.replace('"', '\'')
